{****************************************************************************}
{*                                                                          *}
{*                                 MAKETIEF                                 *}
{*                                                                          *}
{*           Programm zur Erzeugung von Tiefenbildern fr MAGIC-3D          *}
{*              Version 1.0 (c) Rolf Sander & Martin Simeth 1994            *}
{*                                                                          *}
{* Vor dem Programmstart mssen die Konstanten DepthxSize, DepthySize,      *}
{* Picture und Screen auf die gewnschten Werte gesetzt werden.             *}
{*                                                                          *}
{* Zur Eingabe einer neuen Funktion mssen lediglich die beiden             *}
{* case-Anweisungen im Programm erweitert werden.                           *}
{*                                                                          *}
{****************************************************************************}

{$E+} {$N+} {Coprozessor benutzen, falls vorhanden}

program MAKETIEF;

uses
  crt,      {fr ClrScr, ReadKey}
  graph,    {fr PutPixel etc.}
  IOFunct;  {Input- und Outputfunktionen}

const
  DepthxSize = 640;  {Breite des Tiefenbildes}
  DepthySize = 480;  {Hhe des Tiefenbildes}
  Picture    = 12;   {Auswahl der Funktion zur Tiefenberechnung}
  Screen     = true; {true=Bildschirmausgabe, false=Ausgabe in PCX-File}

  PathToDriver = 'C:\MAGIC-3D';    {Pfad des BGI-Grafiktreibers}

type
  MagicLineType   = array[0 .. DepthxSize] of Byte;
  FileOfCharType  = File Of Char; {Bug in Pascal! Typ ohne Leerzeichen}

var
  Alpha              : Real;            {Winkel}
  Depth              : word;            {Tiefe}
  Graphdriver        : integer;
  Graphmode          : integer;
  I                  : Integer;
  Key                : Char;            {gedrckte Taste}
  MagicLine          : MagicLineType;   {eine Zeile des Bildes}
  MaxColors          : Integer;         {16 fr Screen und 256 fr PCX-File}
  OutputFileName     : String;          {Name der Ausgabefiles ohne Suffix}
  OutputID           : FileOfCharType;  {ID des Magicfiles}
  r2Real             : real;
  x                  : word;            {x-Position im Bild}
  xReal              : real;
  y                  : word;            {y-Position im Bild}
  yReal              : real;

{****************************************************************************}
{**********************          OpenOutput          ************************}
{****************************************************************************}

Procedure OpenOutput;
var
  i         : Integer;
  Character : Char;
  Text      : String;

Begin
  Character := Char(10);
  Assign(OutputID,OutputFileName+'.PCX');
  Rewrite(OutputID);
  write(OutputID,Character);
  Character := Char(5);
  Write(OutputID,Character);
  Character := Char(1);
  Write(OutputID,Character);
  Character := Char(8);                           {BitPerPixel}
  Write(OutputID,Character);
  Character := Char(0);
  Write(OutputID,Character);
  Write(OutputID,Character);
  Write(OutputID,Character);
  Write(OutputID,Character);
  Character:= Char((DepthxSize-1) and 255);
  Write(OutputID,Character);
  Character :=  Char((DepthxSize-1) shr 8);
  Write(OutputID,Character);
  Character :=  Char((DepthySize-1) and 255);
  Write(OutputID,Character);
  Character :=  Char((DepthySize-1) shr 8);
  Write(OutputID,Character);
  Character :=  Char(DepthxSize and 255);
  Write(OutputID,Character);
  Character :=  Char(DepthxSize shr 8);
  Write(OutputID,Character);
  Character :=  Char(DepthySize and 255);
  Write(OutputID,Character);
  Character :=  Char(DepthySize shr 8);
  Write(OutputID,Character);

  For i := 0 To 15 Do
  Begin
    Write(OutputID,Char(i));
    Write(OutputID,Char(i));
    Write(OutputID,Char(i));
  end;
  Character := Char(0);
  Write(OutputID,Character);
  Character := Char(1);                                {NPlanes}
  Write(OutputID,Character);
  Character :=  Char(DepthxSize and 255);              {BytesPerLine and 255}
  Write(OutputID,Character);
  Character :=  Char(DepthxSize shr 8);                {BytesPerLine shr 8}
  Write(OutputID,Character);
  Character := Char(1);
  Write(OutputID,Character);
  Character := Char(0);
  Write(OutputID,Character);
  Text := 'MAKETIEF Version 1.0  (c) Rolf Sander & Martin Simeth 1994';
  For i := 1 to 58 Do
  Begin
    Write(OutputID, Text[i]);
  end;

  PrintBox(20,10,60,16,Normal,Framedouble);
  Printxy(22,12,'   MAKETIEF erzeugt '+OutputFileName,normal);
  Printxy(22,14,'       Prozent',normal);

end;

{****************************************************************************}
{*********************           WritePCXLine        ************************}
{****************************************************************************}

Procedure WritePCXLine;
var
  x         : Integer;
  Duplicate : Integer;
  Character : Char;
  Value     : Byte;
  Percent   : String;

Begin
  x := 0;
  while x<=DepthxSize-1 Do
  Begin
    Duplicate := 0;
    Value:=MagicLine[x+Duplicate];
    Duplicate:=Duplicate+1;
    While(Value=MagicLine[Duplicate+x]) and (Duplicate<63)
      and ((x+Duplicate)<=(DepthxSize-1)) Do
    Begin
      Duplicate:=Duplicate+1;
    end;
    If (Duplicate>1) or (Value>=192) Then
    Begin
      Character:=Char(192+Duplicate);
      Write(OutputID,Character);
    end;
      Write(OutputID, Char(Value));
    x:=x+Duplicate;
  end;
  str(Round(y*100./DepthySize),Percent);
  Printxy(25,14,Percent,normal);
end;

{****************************************************************************}
{**********************         CloseOutput          ************************}
{****************************************************************************}

Procedure CloseOutput;
var
  Character : Char;
Begin
  Character := Char(12);
  Write(OutputID,Character);
  For i:=0 To 255 Do
  Begin
    Write(OutputID,Char(I));
    Write(OutputID,Char(I));
    Write(OutputID,Char(I));
  end;
  close(OutputID);
end;

{****************************************************************************}
{*********************         MakeZwergTor          ************************}
{****************************************************************************}

Procedure MakeZwergTor;
const
  rGate0       : Real     = 0.07;
  rGate1       : Real     = 0.35;
  rGate2       : Real     = 0.56;
  yRealHeaven  : Real     = 0.65;
  yRealGateMax : Real     = -0.4;
  yRealGateMin : Real     = -0.7;
  yRealBottom  : Real     = -1.1;
  dGateMax     : Integer  = 255;
  dGateMin     : Integer  = 200;
  dWall        : Integer  = 150;
var
  r : Real;
Begin
  xReal := Abs(xReal);
  If yReal>yRealHeaven Then
  Begin
    {A}
    Depth := 0;
  End Else Begin
    If yReal>yRealGateMax Then
    Begin
      {BCDE}
      r := Sqrt(Sqr(xReal)+Sqr(yReal-yRealGateMax));
      If r>rGate2 Then
      Begin
        {B}
        Depth := dWall;
      End Else Begin
        If r>rGate1 Then
        Begin
          {C}
          Depth := Round(dGateMin + (dGateMax-dGateMin) *
            Sqrt(1-Sqr((2*r-rGate1-rGate2)/(rGate2-rGate1))));
        End Else Begin
          If r>rGate0 Then
          Begin
            {D}
            Depth := Round((r-rGate0)/(rGate1-rGate0)*dWall);
          End Else Begin
            {E}
            Depth := 0;
          End;
        End;
      End;
    End Else Begin
      {FGHIJ}
      r := xReal;
      If xReal>rGate2 Then
      Begin
        {F}
        Depth := dWall;
      End Else Begin
        If xReal>rGate1 Then
        Begin
          {G}
          Depth := Round(dGateMin + (dGateMax-dGateMin) *
            Sqrt(1-Sqr((2*xReal-rGate1-rGate2)/(rGate2-rGate1))));
        End Else Begin
          If (xReal<rGate0) and (yReal>yRealGateMin) Then
          Begin
            {I}
            Depth := 0;
          End Else Begin
            If (xReal<rGate0) or
              ((xReal-rGate0)/(rGate1-rGate0)+
              (yReal-yRealBottom)/(yRealGateMin-yRealBottom)<1) Then
            Begin
              {J}
              Depth := Round(
                dWall*(yReal-yRealGateMin)/(yRealBottom-yRealGateMin));
            End Else Begin
              {H}
              Depth := Round((xReal-rGate0)/(rGate1-rGate0)*dWall);
            End;
          End;
        End;
      End;
    End;
  End;
End;
{****************************************************************************}
{*********************              main              ***********************}
{****************************************************************************}

begin
  If Screen Then
  Begin
    Graphdriver := 0;
    Graphmode := 0;
    InitGraph(GraphDriver,Graphmode,PathToDriver);
    setViewPort(0,0,DepthxSize,DepthySize,true);
    clearviewPort;
    MaxColors:=16;
  end else begin
    ClrScr;
    SetScreenPtr;
    MaxColors:=256;
  end;

  Randomize;

  case Picture of
    1  : OutputFileName := 'RandomBW';   {eher als Muster geeignet !}
    2  : OutputFileName := 'Random';     {dito}
    3  : OutputFileName := 'Trichter';
    4  : OutputFileName := 'Erdball';
    5  : OutputFileName := 'Eikarto1';
    6  : OutputFileName := 'Eikarto2';
    7  : OutputFileName := 'Nichts';
    8  : OutputFileName := 'Splash';
    9  : OutputFileName := 'Pentagon';
    10 : OutputFileName := 'Spirale';
    11 : OutputFileName := 'Spirpent';
    12 : OutputFileName := 'ZwergTor';
    13 : OutputFileName := 'Tiefe';
    14 : OutputFileName := 'Torus';
    15 : OutputFileName := 'Stick';
    16 : OutputFileName := 'Wellen';
  end;{case}
  If not Screen Then OpenOutput;

  for y := 0 to DepthySize do
  Begin
    For x := 0 To DepthxSize do
    Begin

      xReal := 1.1/DepthySize*(2.*x-DepthxSize);
      yReal := 1.1/DepthySize*(DepthySize-2.*y);
      r2Real:=sqr(xReal)+sqr(yReal);
      If (xReal=0) then
        If (yReal<0) then
          Alpha := 3*Pi/2
        else
          Alpha := Pi/2
      else
        Alpha := arctan(yReal/xReal);
      If (xReal<0) then Alpha:=Alpha+Pi;

      case Picture of
        1 :
        begin
          if Random>0.5
            then Depth:=MaxColors-1
          else
            Depth:=0;
        end;
        2 :
        begin
          Depth:=Round(MaxColors*Random);
        end;
        3 :
        begin
          Depth := (MaxColors-1)-Round((MaxColors-1)*Exp(-2*r2Real));
        end;
        4 :
        begin
          if r2Real<0.5 then
            Depth:=round((MaxColors-1)*(0.5+0.5*Sqrt(1-r2Real/0.5)))
          else
            Depth:=0;
        end;
        5 :
        begin
          Depth := (MaxColors shr 1) +
            round(((MaxColors shr 2)-1)*(sin(Pi*xReal)+sin(pi*yReal)));
        end;
        6 :
        begin
          Depth := (MaxColors shr 1) +
            round(((MaxColors shr 1)-1)*(sin(Pi*xReal)*sin(pi*yReal)));
        end;
        7 :
        begin
          Depth := 0;
        end;
        8 :
        begin
          Depth := (MaxColors shr 1) +
            round(Exp(-r2Real)*((MaxColors shr 1)-1)*(cos(3*Pi*Sqrt(r2Real))));
        end;
        9 :
        begin
          Depth := (MaxColors shr 1) + round(
              Exp(-r2Real)
              *((MaxColors shr 1)-1)
              *(cos(3*Pi*Sqrt(r2Real)))
              *(0.6 + 0.4*Sin(5*Alpha))
            );
        end;
        10 :
        begin
          Depth := (MaxColors shr 1) + round(
              4*r2Real*Exp(1-4*r2Real)
              *((MaxColors shr 1)-1)
              *Sin(3*(Alpha+Pi/2*r2Real))
            );
        end;
        11 :
        begin
          Depth := (MaxColors shr 1) + round(
              ((MaxColors shr 1)-1)
              *Cos(2*Pi*Sqrt(r2Real))
              *4*r2Real*Exp(1-4*r2Real)
              *Sin(3*(Alpha+Pi/2*r2Real))
            );
        end;
        12 : MakeZwergTor;
        13 :
        begin
          Depth := (MaxColors shr 1) +
            round(Exp(-Sqr(yReal))*((MaxColors shr 1)-1)*
            (cos(3*Pi*xReal)));
        end;
        14 :
        Begin
          If (r2Real<0.81) and (r2Real>0.16) Then
          Begin
          Depth := (MaxColors shr 1) +Round( ((MaxColors shr 1)-1) *
            Sqrt(1-Sqr((2*Sqrt(r2Real)-0.4-0.9)/(0.9-0.4))));
          End Else Begin
            Depth :=0;
          End;
        End;
        15 :
        Begin
          If (yReal<0.3) and (yReal>-0.3) Then
          Begin
            Depth := Round((MaxColors-1) * x / DepthxSize);
          End Else Begin
            If xReal>0 Then
              Depth :=MaxColors-1
            Else
              Depth :=0;
          End;
        End;
        16 :
        begin
          Depth := Round(0.7*((MaxColors shr 1) +
            ((MaxColors shr 1)-1)*(cos(2*Pi*xReal))));
        end;
      end;{case}
      If Screen Then
        PutPixel(x,y,Depth)
      Else
        MagicLine[x]:=Depth;
    end;

    If not Screen Then WritePCXLine;

  end;

  If Screen Then
  Begin
    repeat
      key := Readkey;
    until (key=Escape);
    closeGraph;
  end else
    CloseOutput;

end.

{****************************************************************************}
{****************************     end of file    ****************************}
{****************************************************************************}
